﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;

using Raven.Client.Document;
using Raven.Client.Linq;
using Raven.Imports.Newtonsoft.Json;

namespace CustomConverters
{
    public class UnitBroker
    {
        public void DeleteUnitWith( Guid unitID )
        {
            using( var session = this.Connection.OpenSession() )
            {
                session.Delete( unitID.ToString() );
                session.SaveChanges();
            }
        }

        public Unit FindUnitWith( Guid requestedUnitID, string perspective = "" )
        {
            IList<UnitDecorator> orgs = null;
            Unit newUnit = null;

            using( var session = this.Connection.OpenSession() )
            {
                orgs = session
                            .Query<UnitDecorator>()
                            .Customize( u => u.Include<UnitDecorator, UnitDecorator>( su => su.UnitIDs ) )
                            .Where( u => u.UnitID == requestedUnitID )                            
                            .ToList();

                if( orgs.Count == 0 )
                {
                    return null;
                }

                newUnit = orgs[0].AsUnit();
                orgs[0].UnitIDs.ForEach( unitID => newUnit.Units.Add( session.Load<UnitDecorator>( unitID ).AsUnit() ) );
            }

            return newUnit;
        }

        public Unit SaveUnit( Unit newOrgTree )
        {
            if( newOrgTree.OID == Guid.Empty )
            {
                newOrgTree.OID = Guid.NewGuid();
            }

            UnitDecorator ud = new UnitDecorator( newOrgTree );

            using( var session = this.Connection.OpenSession() )
            {
                session.Store( ud );
                newOrgTree.Units.ForEach( su => session.Store( new UnitDecorator( su ) ) );
                session.SaveChanges();
            }
            return newOrgTree;
        }

        #region Private Properties
        private DocumentStore Connection
        {
            get
            {
                return c_Connection;
            }
        }
        #endregion

        #region Construction and Finalization
        public UnitBroker( string endpoint, string key, string databaseName )
        {
            if( c_Connection == null )
            {
                c_Connection = new DocumentStore();
                this.Connection.Url = endpoint.ToString();
                this.Connection.DefaultDatabase = databaseName;
                this.Connection.Conventions.AcceptGzipContent = true;
                this.Connection.Conventions.AllowQueriesOnId = true;
                this.Connection.Conventions.CustomizeJsonSerializer = serializer => {
                                                                                        serializer.TypeNameHandling = TypeNameHandling.All;
                                                                                        serializer.Converters.Add( new UnitConverter( ConfigurationManager.AppSettings, typeof( Unit ) ) );
                                                                                    };
                this.Connection.Initialize();
            }
        }
        #endregion

        #region Data Elements
        private static DocumentStore c_Connection;
        #endregion
    }
}

