﻿using System;
using System.Collections.Specialized;
using System.Linq;

using Raven.Imports.Newtonsoft.Json;
using Raven.Imports.Newtonsoft.Json.Linq;

namespace CustomConverters
{
    public class UnitConverter : JsonConverter
    {
        #region Constants
        #endregion

        #region Event Handlers
        #endregion

        #region Methods
        public override bool CanConvert( Type nextStreamType )
        {
            bool shouldHandle = i_ManagedTypes.Any( managedType => managedType == nextStreamType );
            return shouldHandle;
        }

        public override object ReadJson( JsonReader stream, Type objectType, object existingValue, JsonSerializer serializer )
        {
            UnitConverter privateConverter = new UnitConverter( this.Settings, this.ManagedTypes );
            return privateConverter.DeserializeFrom( stream );
        }

        /// <summary>
        /// Save each Unital unit as a separate entity, if necessary, and then store the parent structure.
        /// </summary>
        /// <param name="stream"></param>
        /// <param name="value"></param>
        /// <param name="serializer"></param>
        public override void WriteJson( JsonWriter stream, object value, JsonSerializer serializer )
        {
            Unit org = value as Unit;
            UnitBroker broker = this.GetBroker();

            org.Units.ForEach( subunit => broker.SaveUnit( subunit ) );

            stream.WriteStartObject();

            this.WriteClassProperties( stream, org, serializer );

            stream.WritePropertyName( "PrimUnits" );
            stream.WriteStartArray();
            org.Units.ForEach( subunit => this.WriteUnitReferenceTo( stream, subunit ) );
            stream.WriteEndArray();

            this.WriteSuperClassProperties( stream, org, serializer );

            stream.WriteEndObject();
        }

        #endregion

        #region Properties
        public override bool CanRead
        {
            get
            {
                return true;
            }
        }

        public override bool CanWrite
        {
            get
            {
                return true;
            }
        }
        #endregion

        #region Private Methods
        private Unit DeserializeFrom( JsonReader stream )
        {
            JObject streamElements = JObject.Load( stream );
            stream.Close();
            Unit org = new Unit( streamElements["Name"].Value<string>(),
                                 new Guid( streamElements["OID"].Value<string>() ) );

            UnitBroker broker = this.GetBroker();

            foreach( JProperty referenceID in streamElements["PrimUnits"].Values() )
            {
                org.Units.Add( broker.FindUnitWith( new Guid( referenceID.Value.Value<string>() ) ) );
            }

            return org;
        }

        private UnitBroker GetBroker()
        {
            return new UnitBroker( this.Settings["DatabaseServer"],
                                           this.Settings["Key"],
                                           this.Settings["Database"] );
        }
        private void WriteClassProperties( JsonWriter stream, Unit unit, JsonSerializer serializer )
        {
            stream.WritePropertyName( "$type" );
            serializer.Serialize( stream, this.TypeName );
            stream.WritePropertyName( "Name" );
            serializer.Serialize( stream, unit.Name );
        }

        private void WriteUnitReferenceTo( JsonWriter stream, Unit subunit )
        {
            stream.WriteStartObject();
            stream.WritePropertyName( "referenceID" );
            stream.WriteValue( subunit.OID );
            stream.WriteEndObject();
        }

        private void WriteSuperClassProperties( JsonWriter stream, Unit unit, JsonSerializer serializer )
        {
            stream.WritePropertyName( "OID" );
            serializer.Serialize( stream, unit.OID );
        }
        #endregion

        #region Private Properties
        private Type[] ManagedTypes
        {
            get
            {
                return i_ManagedTypes;
            }
            set
            {
                i_ManagedTypes = value;
                string[] sections = i_ManagedTypes[0].AssemblyQualifiedName.Split( ',' );
                this.TypeName = $"{sections[0]}, {sections[1]}";
            }
        }

        private NameValueCollection Settings
        {
            get
            {
                return i_Settings;
            }
            set
            {
                i_Settings = value;
            }
        }

        private string TypeName
        {
            get
            {
                return this.i_TypeName;
            }
            set
            {
                if( string.IsNullOrWhiteSpace( value ) )
                {
                    throw new ArgumentNullException( "Type name of the type handled by this converter cannot be empty or null. Check to ensure you constructed the converter correctly." );
                }
                this.i_TypeName = value;
            }
        }
        #endregion

        #region Construction and Finalization
        public UnitConverter( NameValueCollection settings, params Type[] types )
        {
            this.ManagedTypes = types;
            this.i_Settings = settings;
        }
        #endregion

        #region Data Elements
        private Type[] i_ManagedTypes;
        private NameValueCollection i_Settings;
        private string i_TypeName;
        #endregion
    }
}


